﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LdapLoginLib.Models
{
    public class LdapUserInfo
    {
        /// <summary>
        /// The unique identifier for the user (mandatory).
        /// Example: "jdoe"
        /// </summary>
        public string? Uid { get; set; }

        /// <summary>
        /// Doc. Identidad
        /// The employee's unique identification number.
        /// Example: "E12345"
        /// </summary>
        public string? EmployeeNumber { get; set; }

        /// <summary>
        /// The common name of the user.
        /// Example: "John Doe"
        /// </summary> 
        public string? Cn { get; set; }

        /// <summary>
        /// Nombres full
        /// The user's given name.
        /// Example: "John"
        /// </summary>
        public string? GivenName { get; set; }

        /// <summary>
        /// Apellidos
        /// The user's surname.
        /// Example: "Doe"
        /// </summary>
        public string? Sn { get; set; }

        /// <summary>
        /// The user's email address.
        /// Example: "jdoe@example.com"
        /// </summary>
        public string? Mail { get; set; }


        //TODO:
        /// <summary>
        /// The user's ALTERNATIVE email address.
        /// Example: "jdoe@example.com"
        /// </summary>
        public string? MailAlternateAddress { get; set; }


        /// <summary>
        /// The organization the user belongs to.
        /// Example: "Acme Inc.", currently "Sede"
        /// </summary>
        public string? O { get; set; }

        /// <summary>
        /// The status of the user's internet account.
        /// Example: "Active"
        /// </summary>
        public string? InetUserStatus { get; set; }

        /// <summary>
        /// The status of the user's account as boolean.
        /// Example: true or false
        /// </summary>
        public bool? IsActive { get; set; } = null;




    }


    /********************************************
     *                                          * 
     *          Discared / not in used          *
     *                                          *
     ********************************************
        
        /// <summary>
        /// The user's password.
        /// Example: "P@ssw0rd"
        /// </summary>
        public string? UserPassword { get; set; }

        /// <summary>
        /// The type of employee (e.g., full-time, part-time).
        /// Example: "Full-Time", currently numbers 
        /// </summary>
        //public string? EmployeeType { get; set; }


        /// <summary>
        /// The business category of the user.
        /// Example: "Sales"
        /// </summary>
        public string? BusinessCategory { get; set; }

        // Definidos por LDAP
        // No encontrado
        /// <summary>
        /// Numero unico identificacion personal
        /// </summary>
        //public string? NUIP { get; set; } = null;

        /// <summary>
        /// The license information for the user.
        /// Example: "Licensed for Software X, Y, and Z"
        /// </summary>
        public string? NsLicensedFor { get; set; }


    ********************************************
    *                                          * 
    ********************************************/

}
